/** \file ES_01_01.cc
 * <PRE>
 * File Type   : C++ Source Code
 * Begin       : 2005-01-17
 * Last Update : 2005-01-18
 * Description : Calcolo valori eps (precisione macchina)
 * Platform    : 386+ PC / 32bit DOS
 * Compiler    : DJGPP (http://www.delorie.com/djgpp/)
 * </PRE>
 * NOTE: compilare il software in CPP con le seguenti opzioni 
 * per evitare che il compilatore utilizzi precisione extra:
 * -ffloat-store
 * -fsingle-precision-constant
 * -mieee-fp
 * -mfpmath=sse
 * -msse
 * -msse2
 * @author Nicola Asuni [38433]
 */
 
#ifndef __dj_include_stdio_h_
#include <stdio.h>
#endif

#ifndef __dj_include_conio_h_
#include <conio.h>
#endif

#ifndef __dj_include_math_h_
#include <math.h>
#endif

#define round(x) ((int)(x + 0.5))

/**
 * Template class to get floating point system information.<BR>
 */
template<class T> class FPInfo {
	private:
	
	public:
		/**
		 * Class constructor.
		 */
		FPInfo(void) {
		};
		
		
		/**
		 * Class destructor.
		 */
		~FPInfo(void) {
		};
		
		/**
		 * Restituisce la precisione eps del compilatore.
		 * @return float eps
		 */
		T getEps(void){
			T eps;
			eps = 1;
			while ((eps + 1) > 1) {
				eps *= 0.5;
			}
			return(2 * eps);
		}
		
		/**
		 * Restituisce il numero di cifre significative
		 * @return int t
		 */
		int getT(){
			T eps = getEps();
			return round(1 - log10(2 * eps));
		}
		
		/**
		 * Restituisce L. 
		 * L  l'esponente pi piccolo per il quale la base  diversa da 0.
		 * @return int L
		 */
		int getL(){
			T base = 10;
			int L = 0;
			while (base > 0) {
				base = base / 10; // faccio decrescere rapidamente la base
				L = L - 1;
			}
			return L;
		}
		
		/**
		 * Restituisce U. 
		 * U  il massimo esponente che pu assumere la base prima di un overflow.
		 * @return int U
		 */
		int getU(){
			T base = 10;
			int U = 0;
			while (base < HUGE_VAL) {
				base = base * 10; // faccio crescere rapidamente la base
				U = U + 1;
			}
			return U;
		}
		
};


int main(int argc, char *argv[]) {
	
	clrscr(); // clear screen
		
	FPInfo<float> fp_float;
	FPInfo<double> fp_double;
	
	printf("\r\nInformazioni sul sistema di Floating Point del calcolatore:\r\n\r\n");
	printf("[float]  eps = %g\r\n", fp_float.getEps());
	printf("[double] eps = %g\r\n", fp_double.getEps());
	printf("[float]    t = %d\r\n", fp_float.getT());
	printf("[double]   t = %d\r\n", fp_double.getT());
	printf("[float]    L = %d\r\n", fp_float.getL());
	printf("[double]   L = %d\r\n", fp_double.getL());
	printf("[float]    U = %d\r\n", fp_float.getU());
	printf("[double]   U = %d\r\n", fp_double.getU());
	getch();
		
	return 0;
}

/* END OF FILE */
