clc; % clear command window
clear; % frees system memory
echo on; % turns on the echoing of commands in all script files
% =========================================================================
% File name   : ES_13.m
% File Type   : m-file (script file for matlab or octave)
% Begin       : 2005-01-18
% Last Update : 2005-01-19
% Author      : Nicola Asuni [38433]
% Description : Esercitazione di Analisi Numerica
%               A.A. 2004-2005
%               Prof. M. Gaviano
%               UNIVERSIT DEGLI STUDI DI CAGLIARI
%               Facolt di Scienze Matematiche Fisiche Naturali
%               Dipartimento di Matematica ed Informatica
%               Corso di Laurea in Informatica
% =========================================================================
echo off;

disp('Risoluzione Ax=b con Jacobi e Gauss-Seidel');
        
n = input('Inserisci la dimensione del sistema: ');
itmax = input('Inserisci il numero massimo di interazioni: ');
acc = input('Inserisci l''accuratezza richiesta: ');

if isnumeric(n) & (n > 0) & isnumeric(itmax) & (itmax > 0) & isnumeric(acc) & (acc > 0)
    % calcolo una nuova matrice finch il det(A) = 0 e finch l'utente
    % vuole continuare 
    reply = 's'; % risposte utente
    while strcmpi(reply, 's')
        reply = 'n';
        disp('genero una matrice casuale convergente per Jacobi');
        B = (1/n) * rand(n);
        DD = diag(diag(B));
        B = B - DD;
        A = DD - DD * B
        
        % il determinante di una matrice non da buona indicazione del suo
        % malcondizionamento. Verifico la singolarit della matrice con
        % cond(A) in quanto il test su det(A)  appropriato solo per 
        % matrici modeste di interi di piccolo valore.
        
        if (i == 3) & (cond(A) >= 1/eps)
            % chiede all'utente se generare una nuova matrice
            disp('la matrice  mal condizionata');
            reply = input('genero una nuova matrice? S/N [N]: ','s');
        else
            
            for i = 1:3
                % matrici di test
                switch i
                    case 2
                        disp('---------- TEST 2 ----------');
                        A = [1 -2 2; -1 1 -1; -2 -2 1]
                        n = 3;
                    case 3
                        disp('---------- TEST 3 ----------');
                        A = [2 -1 1; 2 2 2; -1 -1 2]
                        n = 3;
                end
                disp('genero la soluzione esatta del problema');
                xx = rand(n,1)

                disp('calcolo il vettore dei termini noti');
                b = A * xx

                disp('calcolo la soluzione approssimata con Jacobi');
                x = jacobi(A,b,acc,itmax);
                
                if (isnan(x))
                    disp('ATTENZIONE: Il sistema non  convergente!');
                else
                    x = x
                    % Stampa l'errore di approssimazione con la norma euclidea
                    disp(sprintf('L''errore commesso  (norma euclidea): %d\n', norm(xx-x)));
                end
                disp('calcolo la soluzione approssimata con Gauss-Seidel');
                x = gaussSeidel(A,b,acc,itmax);

                if (isnan(x))
                    disp('ATTENZIONE: Il sistema non  convergente!');
                else
                    x = x
                    % Stampa l'errore di approssimazione con la norma euclidea
                    disp(sprintf('L''errore commesso  (norma euclidea): %d\n', norm(xx-x)));
                end
                
            end
        end
   end
else
   disp('ERRORE NEI DATI DI INPUT!'); 
end

% -------------------------------------------------------------------------
% NOTE DI TEORIA
% -------------------------------------------------------------------------
%
% 1. Metodi iterativi per la risoluzione di un sistema lineare
% Determinano la soluzione x* come limite (punto di acumulazione) di una 
% successione x* = lim i?? xi ;  Ax*=b
% La soluzione pu essere sufficientemente approssimata dopo un numero 
% piccolo di iterazioni. 
% Se la matrice  sparsa, questa caratteristica non viene persa. 
% Si utilizzano per affinare le soluzioni ottenute con i metodi diretti.
% 
% 2. Schema iterattivo metodi Jacobi e Gauss-Seidel
% Posto Ax=b, A matrice nxn, det(A)!=0, b appatenente ad R^n,
% A si scompone nelle matrici M ed N: A = M  N, con det(M)!=0
% Ax=b => Mx = Nx +b ; Mx^(k+1) = Nx^(k) +b
% schema delle interazioni: x^(k+1) = inv(M)Nx^(k) + inv(M)b
% matrice di iterazione: B = inv(M)N = inv(M)(M  A) = I - inv(M)A
%
% 3. schema iterativo di Jacobi
% Si pone: A=D-E-F; M=D; N=E+F; Bj=(E+F)=I-inv(D)A
%
% 4. schema iterativo di Gauss-Seidel
% Si pone: A=D-E-F; M=D-E; N=E; BG=inv(D-E)=inv(I-inv(D)E)F
%
% 5. Convergenza del metodo basato sullo schema Mx(k+1) = Nx(k) + b
% Il metodo iterativo definito dalla matrice di iterazione B, converge per 
% ogni punto iniziale x0 di Rn se e solo se il raggio spettrale della 
% matrice di iterazione B= M'N  minore di 1: ro(B) < 1.
% Infatti: x* = Bx*+ inv(M)b   ;  x^(k+1) = Bx^(k) + inv(M)b
% indicando l'errore con e^(k+1) = x* - x^(k) ; 
% e^(k+1) = Be(k) => e^(k+1) = B^k e^(0)
% se B  una matrice convergente allora ||e^(k)||->0  per  k->inf

% === EOF =================================================================