clc; % clear command window
clear; % frees system memory
echo on; % turns on the echoing of commands in all script files
% =========================================================================
% File name   : ES_17_02.m
% File Type   : m-file (script file for matlab or octave)
% Begin       : 2005-01-18
% Last Update : 2005-01-19
% Author      : Nicola Asuni [38433]
% Description : Esercitazione di Analisi Numerica
%               A.A. 2004-2005
%               Prof. M. Gaviano
%               UNIVERSIT DEGLI STUDI DI CAGLIARI
%               Facolt di Scienze Matematiche Fisiche Naturali
%               Dipartimento di Matematica ed Informatica
%               Corso di Laurea in Informatica
% =========================================================================
echo off;

disp('Polinomio interpolante - minimi quadrati - errore casuale');

m = input('Inserisci il numero di nodi m: ');
n = input('Inserisci il grado del polinomio n < m: ');
p = input('Inserisci il numero di punti per il grafico: ');

if isnumeric(m) & (m > 1) & isnumeric(n) & (n > 0) & (n < m) & isnumeric(p) & (p > 0)      
    
    for i = 1:5
       % eseguo test con funzioni diverse
        switch i
            case 1
                disp('---------- TEST 1: y = 2sen(x)+2 ----------');
                a = -20;
                b = 20;
                x = linspace(a,b,m)';
                xs = linspace(a,b,p)';
                y = fun5(x);
                fxs = fun5(xs);
            case 2
                disp('---------- TEST 2: y = exp(-x)-sin(2 pi x) ----------');
                a = 0;
                b = 4;
                x = linspace(a,b,m)';
                xs = linspace(a,b,p)';
                y = fun6(x);
                fxs = fun6(xs);
           case 3
                disp('---------- TEST 3: y = (-3x+1.4)*sin(18x) ----------');
                a = 0.96609;
                b = 1.48907;
                x = linspace(a,b,m)';
                xs = linspace(a,b,p)';
                y = fun7(x);
                fxs = fun7(xs);
           case 4
                disp('---------- TEST 4: y = 1/(1+x^2)  (runge) ----------');
                a = -6;
                b = 6;
                x = linspace(a,b,m)';
                xs = linspace(a,b,p)';
                y = fun4(x);
                fxs = fun4(xs);
           case 5
                disp('---------- TEST 5: y = 1/(1+x^2)  (runge + chebichev) ----------');
                a = -6;
                b = 6;
                x = chebichev(a,b,n)';
                xs = linspace(a,b,p)';
                y = fun4(x);
                fxs = fun4(xs);
        end
        
        % introduco un errore casuale del 10%
        error_perc = 10; % percentuale di errore da introdurre 
        y = y-(y/error_perc)+((y/(error_perc/2)).*rand(length(x),1));
        fxs = fxs-(fxs/error_perc)+((fxs/(error_perc/2)).*rand(p,1));
        
        % interpolo tramite spline lineare
        mxs = minQuadPoly(xs,x,y,n);
        
        % faccio i grafici sovrapposti
        plot(x,y,'o',xs,fxs,xs,mxs,'*-');
        legend('y','fxs','mxs');
        
        reply = input('Premi [INVIO] per continuare');
    end
else
   disp('ERRORE NEI DATI DI INPUT!'); 
end

% -------------------------------------------------------------------------
% NOTE DI TEORIA
% -------------------------------------------------------------------------
%
% 1. Migliore approssimazione
% Si cerca quella funzione (in una determinata famiglia di funzioni) la cui
% distanza dalla funzione da approssimare  minima.
% dist(f,g) = ||f(x)-g(x)||inf = max[a<=x<=b](|f(x)-g(x)|)
%
% 2. Formulazione del problema di migliore approssimazione.
% Dato uno spazio lineare normato V e un sottospazio W di dimensione n+1 e 
% generato dagli elementi F0,F1,,Fn di V (costituiscono una base), ovvero
% ogni elemento w di W  dato da w=Sum[j=0,n](ajFj).
% Assegnato un elemento qualsiasi f di V occorre determinare un elemento w*
% tale che: ||f-w*||<||f-w|| per ogni w* appartenente a W.
%
% 3. Esempio di problema di migliore approssimazione.
%  - V=C[a,b], l'insieme delle funzioni continue in [a,b]
%  - W=Pn(x), l'insieme dei polinomi di grado <= n avente per base 
%    1,x,x^2,...,x^n
%  - la norma utilizzata  quella del massimo ||.||[Inf]
% Data una qualsiasi funzione f(x) in C[a,b], si cerca in Pn(x) il 
% polinomio p(x) pi vicino a f(x) secondo la distanza indotta dalla norma 
% del massimo.
%
% 4. Migliore approssimazione nel senso dei minimi quadrati nel continuo
% E un caso particolare di miglior approssimazione
%  - V=L^2[a,b], insieme delle funzioni f(x) in [a,b] per le quali 
%    l'integrale di (f(x))^2 su [a,b] esiste ed  finito
%  - W sottospazio di V di dimensione finita. Per esempio W=Pn(x), 
%    l'insieme dei polinomi di grado <= n
%  - la norma utilizzata  la norma 2, ovvero per f appartenente a V
%    ||f(x)||2=(Integrale[a,b](f(x))^2 dx)^1/2 
%
% 5. Migliore approssimazione nel senso dei minimi quadrati nel discreto.
% E un caso particolare di miglior approssimazione
%  - V= insieme delle funzioni f(x) per cui esiste una tabulazione 
%    {f(x1),f(x2),...,f(xm)} su S={x1,x2,...,xm} nell'intervallo [a,b]
%  - W sottospazio di V di dimensione finita. Per esempio W=Pn(x), 
%    l'insieme dei polinomi di grado <= n
%  - la norma utilizzata  la seminorma ||.||[2,s] , ovvero per f 
%    appartenente a V
%    ||f(x)||[2,s]=(Sum[i=1,m](f(x))^2)^1/2
%
% 6. TEOREMA
% Sia V uno spazio lineare normato e W un suo sottospazio lineare di
% dimensione finita. Per ogni f di V esiste un elemento di miglior
% approssimazione.
% L'unicit si ha se V  strettamente normato.

% === EOF =================================================================