function xz = chebichev(a,b,n)
    % =====================================================================
    % File name   : cholesky.m
    % File Type   : m-file (script file for matlab or octave)
    % Begin       : 2005-01-18
    % Last Update : 2005-01-20
    % Author      : Nicola Asuni [38433]
    % Description : Esercitazione di Analisi Numerica
    %               A.A. 2004-2005
    %               Prof. M. Gaviano
    %               UNIVERSIT DEGLI STUDI DI CAGLIARI
    %               Facolt di Scienze Matematiche Fisiche Naturali
    %               Dipartimento di Matematica ed Informatica
    %               Corso di Laurea in Informatica
    % =====================================================================
   
    % Calcola gli zeri del polinomio di Chebichev
    % prende in ingresso l'intervallo [a,b] e il grado del polinomio
    % di interpolazione della funzione.
    % Restituisce gli n+1 punti in cui si annulla il polinomio di Chebichev

    for i = 1:n+1
        xz(n+2-i) = ((a+b)./2)+((b-a)./2).*cos((pi.*((2.*i)-1))./(2.*(n+1)));
    end
    
% -------------------------------------------------------------------------
% NOTE DI TEORIA
% -------------------------------------------------------------------------
% 1. Polinomi di Chebichev
% T0=1, T1=x, Tn+1(x)=2xTn(x)-Tn-1(x)
% Si considerano nell'intervallo [-1,1]
% Sono ortogonali rispetto alla seguente definizione di prodoto scalare:
% Integrale[-1,1]{(1-x^2)^(-1/5) f(x)g(x)dx} con f(x) e gx) funzioni su
% [-1,1].
%
% 2. Teorema di BERNSTEIN
% Se f(x) appartiene alle funzioni continue derivabili in [a,b], allora il
% polinomio Pn(f) di grado n di interpolazione di f(x) e relativo agli zeri
% del polinomio di Chebichev di grado n+1 converge uniformemente a f(x) su
% [a,b] per n che tende ad infinito.
%
% Le ascisse di Cebyshev sono definite nell'intervallo [-1,1], quindi per 
% riportare le ascisse ad un generico intervallo [a,b] si usa la formula: 
% X=((a+b)/2)+((b-a)/2)x

% === EOF =================================================================

